@@ -1,16 +1,19 @@
 from __future__ import annotations
+from dataclasses import dataclass
+
 from functools import reduce
-from typing import Callable, Optional, cast
+from typing import Callable, Optional, Union, cast
 from typing_extensions import Literal
+
 import penman
 from penman.tree import Tree, Node
 from penman.graph import Graph
+
 from amr_logic_converter.AmrContext import (
     AmrContext,
     OverrideInstanceScopeCallback,
     OverrideInstanceScopeCallbackMeta,
 )
-
 from amr_logic_converter.types import (
     And,
     Constant,
@@ -22,7 +25,39 @@ from amr_logic_converter.types import (
     Variable,
 )
 
-INITIAL_CLOSURE: Callable[[str], Literal[True]] = lambda u: True
+
+@dataclass
+class OverrideQuantificationCallbackMeta:
+    """Metadata passed to the OverrideQuantificationCallback wtih info about the node being processed"""
+
+    instance_name: str
+    bound_instance: Variable | Constant
+    node: Node
+    amr_tree: Tree
+    is_negated: bool
+
+
+OverrideQuantificationCallback = Callable[
+    [Formula, OverrideQuantificationCallbackMeta], Union[Formula, None]
+]
+
+
+@dataclass
+class OverrideConjunctionCallbackMeta:
+    """Metadata passed to the OverrideConjunctionCallback wtih info about the node being processed"""
+
+    predicate_term: Predicate
+    closure_term: Formula | None
+    subterms: list[Formula]
+    instance_name: str
+    bound_instance: Variable | Constant
+    node: Node
+    amr_tree: Tree
+
+
+OverrideConjunctionCallback = Callable[
+    [Formula, OverrideConjunctionCallbackMeta], Union[Formula, None]
+]
 
 
 def normalize_predicate(predicate: Predicate) -> Predicate:
@@ -60,6 +95,8 @@ class AmrLogicConverter:
     maximally_hoist_coreferences: bool
     capitalize_variables: bool
     override_instance_scope: Optional[OverrideInstanceScopeCallback]
+    override_quantification: Optional[OverrideQuantificationCallback]
+    override_conjunction: Optional[OverrideConjunctionCallback]
 
     def __init__(
         self,
@@ -69,6 +106,8 @@ class AmrLogicConverter:
         maximally_hoist_coreferences: bool = False,
         capitalize_variables: bool = True,
         override_instance_scope: Optional[OverrideInstanceScopeCallback] = None,
+        override_quantification: Optional[OverrideQuantificationCallback] = None,
+        override_conjunction: Optional[OverrideConjunctionCallback] = None,
     ) -> None:
         self.invert_relations = invert_relations
         self.capitalize_variables = capitalize_variables
@@ -76,6 +115,8 @@ class AmrLogicConverter:
         self.use_variables_for_instances = use_variables_for_instances
         self.maximally_hoist_coreferences = maximally_hoist_coreferences
         self.override_instance_scope = override_instance_scope
+        self.override_quantification = override_quantification
+        self.override_conjunction = override_conjunction
 
     def _get_bound_instance(self, instance_name: str) -> Variable | Constant:
         use_variables_for_instances = (
@@ -97,14 +138,14 @@ class AmrLogicConverter:
         node: Node,
         closure: Optional[Callable[[str], Formula | None]] = None,
     ) -> Formula | None:
-        instance_name, instance_info = node
-        instance_predicate, *edges = instance_info
-        is_projective_instance = ctx.is_instance_projective(instance_name)
         # handle 7.2, 7.6-7.8 from "Expressive Power of Abstract Meaning Representations"
         # ∥x,φ∥↓ = φ(x)
         # ∥(x\P),φ∥↓ = φ(x)
         # ∥(x\P :RiAi),φ∥↓ = φ(x)
         # ∥(x\P :RiAi :polarity–),φ∥↓ = φ(x)
+        instance_name, instance_info = node
+        instance_predicate, *edges = instance_info
+        is_projective_instance = ctx.is_instance_projective(instance_name)
         if is_projective_instance:
             return None if closure is None else closure(instance_name)
         bound_instance = self._get_bound_instance(instance_name)
@@ -124,8 +165,8 @@ class AmrLogicConverter:
                     else predicate
                 )
 
-            # special case for the :polarity - attribute. When this is present,
-            # the attribute should be removed but the entire expression should be negated
+            # special case for the :polarity - attribute.
+            # skip polarity as is handled in quantification
             if role == ":polarity" and target == "-":
                 continue
             elif type(target) is tuple:
@@ -143,6 +184,20 @@ class AmrLogicConverter:
                     else predicate
                 )
 
+        if self.override_conjunction is not None:
+            meta = OverrideConjunctionCallbackMeta(
+                predicate_term=predicate_term,
+                closure_term=closure_term,
+                subterms=sub_terms,
+                instance_name=instance_name,
+                bound_instance=bound_instance,
+                node=node,
+                amr_tree=ctx.amr_tree,
+            )
+            override_result = self.override_conjunction(predicate_term, meta)
+            if override_result is not None:
+                return override_result
+
         pre_terms = []
         if closure_term is not None:
             pre_terms.append(closure_term)
@@ -163,6 +218,19 @@ class AmrLogicConverter:
                 polarity = False
 
         def quantification_closure(clause: Formula) -> Formula:
+            if self.override_quantification is not None:
+                override_expr = self.override_quantification(
+                    clause,
+                    OverrideQuantificationCallbackMeta(
+                        node=node,
+                        instance_name=instance_name,
+                        bound_instance=bound_instance,
+                        amr_tree=ctx.amr_tree,
+                        is_negated=not polarity,
+                    ),
+                )
+                if override_expr is not None:
+                    return override_expr
             expr = clause
             if self.existentially_quantify_instances:
                 expr = Exists(cast(Variable, bound_instance), body=clause)
